import ij.*;
import ij.plugin.PlugIn;
import ij.process.*;
import ij.gui.*;

public class RGB_Stack_Merge implements PlugIn {

	private ImagePlus imp;

	public void run(String arg) {
		imp = WindowManager.getCurrentImage();
		MergeStacks();
	}

	/** Combines three grayscale stacks into one RGB stack. */
	public void MergeStacks() {
		int[] wList = WindowManager.getIDList();
		if (wList==null) {
			IJ.error("No images are open.");
			return;
		}
		if (wList.length!=3) {
			IJ.error("There must be exactly three stacks open.");
			return;
		}
		ImagePlus[] image = new ImagePlus[wList.length];
		for (int i=0; i<wList.length; i++) {
			image[i] = WindowManager.getImage(wList[i]);
			int size = image[i].getStackSize();
			if (size==1) {
				IJ.error("All open images must stacks.");
				return;
			}
		}
		for (int i=0; i<(wList.length-1); i++) {
			if (image[i].getType()!=ImagePlus.GRAY8) {
				IJ.error("All stacks be 8-bit grayscale.");
				return;
			}
			if (image[i].getWidth()!=image[i+1].getWidth()
			|| image[i].getHeight()!=image[i+1].getHeight()) {
				IJ.error("All stacks must have the same width and height.");
				return;
			}
		}

		boolean okay = IJ.showMessageWithCancel("RGB Stack Merge", 
			"This command combines three 8-bit stacks into\n"
			+ "one RGB stack.\n \n"
			+ "Note that \"Undo\" is not supported and parts\n"
			+ "of the input stacks will be missing if ImageJ\n"
			+ "runs out of memory during the conversion."
			);
		if (!okay)
			return;

		ImageStack red = image[0].getStack();
		ImageStack green = image[1].getStack();
		ImageStack blue = image[2].getStack();
		ImageStack rgb = MergeStacks(red, green, blue);
		for (int i=0; i<wList.length; i++) {
			image[i].changes = false;
			ImageWindow win = image[i].getWindow();
			if (win!=null)
				win.close();
		}
		new ImagePlus("RGB", rgb).show();
	}
	
	public ImageStack MergeStacks(ImageStack red, ImageStack green, ImageStack blue) {
		int nSlices = 9999;
		int width = red.getWidth();
		int height = red.getHeight();
		if (red.getSize()<nSlices) nSlices = red.getSize();
		if (green.getSize()<nSlices) nSlices = green.getSize();
		if (blue.getSize()<nSlices) nSlices = blue.getSize();
		ImageStack rgb = new ImageStack(width, height);
	    int inc = nSlices/10;
	    if (inc<1) inc = 1;
	    red.setSlice(1); green.setSlice(1); blue.setSlice(1);
	    ColorProcessor cp;
		try {
	    	for (int i=1; i<=nSlices; i++) {
				cp = new ColorProcessor(width, height);
	    		cp.setRGB((byte[])red.getPixels(), (byte[])green.getPixels(), (byte[])blue.getPixels());
	    		red.deleteSlice(1); green.deleteSlice(1); blue.deleteSlice(1);
				System.gc();
				rgb.addSlice("", cp);
				if ((i%inc) == 0) IJ.showProgress((double)i/nSlices);
	    	}
			IJ.showProgress(1.0);
		} catch(OutOfMemoryError o) {
			IJ.outOfMemory("MergeStacks");
			IJ.showProgress(1.0);
		}
		return rgb;
	}

}

